/*
 * Copyright 2012-2022 THALES.
 *
 * This file is part of AuthzForce CE.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ow2.authzforce.core.pdp.impl;

import oasis.names.tc.xacml._3_0.core.schema.wd_17.AttributeDesignatorType;
import org.ow2.authzforce.core.pdp.api.AttributeFqn;
import org.ow2.authzforce.core.pdp.api.value.Datatype;

import javax.annotation.concurrent.Immutable;

/**
 * Immutable version of {@link AttributeDesignatorType}
 */
@Immutable
public final class ImmutableAttributeDesignator extends AttributeDesignatorType
{
    private static final long serialVersionUID = 1L;

    /**
     * Similar to {@link AttributeDesignatorType#AttributeDesignatorType(String, String, String, String, boolean)}
     * @param attributeName attribute FQN (Category, AttributeId, Issuer)
     * @param datatype attribute datatype
     * @param mustBePresent MustBePresent
     */
    public ImmutableAttributeDesignator(final AttributeFqn attributeName, final Datatype<?> datatype, final boolean mustBePresent)
    {
        super(attributeName.getCategory(),  attributeName.getId(), datatype.getId(), attributeName.getIssuer().orElse(null), mustBePresent);
    }
}
